<?php
/**
 * Functions copied from: Magento 1.9.1.0
 *
 * Reason: Due to bugs in Magento, once an rewrite URL ends with -[number] you get more and more rewrite URLs
 *         to the same target. The number gets higher and higher.
 *
 * Changes marked by:  // PATCH
 */

/**
 * Model for overriding core URL functions
 *
 * This fix is developed by Bas Peters from Comaxx BV, The Netherlands.
 * It was packaged by MagentoHosting.pro / MageHost BVBA.
 */
class MageHost_RewriteFix_Model_Catalog_Url extends Mage_Catalog_Model_Url {

    /**
     * Get unique category request path
     *
     * @param Varien_Object $category
     * @param string $parentPath
     * @return string
     */
    public function getCategoryRequestPath($category, $parentPath)
    {
        if ( version_compare( Mage::getVersion(), '1.7.0.0', '<' ) ) {
            // The fix below is not compatible withR older Magento versions.
            // Then only the shell command of this extension is useful.
            return parent::getCategoryRequestPath($category, $parentPath);
        } else {
            $storeId = $category->getStoreId();
            $idPath  = $this->generatePath( 'id', null, $category );
            $suffix  = $this->getCategoryUrlSuffix( $storeId );

            if (isset( $this->_rewrites[ $idPath ] )) {
                $this->_rewrite = $this->_rewrites[ $idPath ];
                $existingRequestPath = $this->_rewrites[ $idPath ]->getRequestPath();
            }

            if ($category->getUrlKey() == '') {
                $urlKey = $this->getCategoryModel()->formatUrlKey( $category->getName() );
            } else {
                $urlKey = $this->getCategoryModel()->formatUrlKey( $category->getUrlKey() );
            }

            $categoryUrlSuffix = $this->getCategoryUrlSuffix( $category->getStoreId() );
            if (null === $parentPath) {
                $parentPath = $this->getResource()->getCategoryParentPath( $category );
            } elseif ($parentPath == '/') {
                $parentPath = '';
            }
            $parentPath = Mage::helper('catalog/category')->getCategoryUrlPath($parentPath,
                                                                           true, $category->getStoreId());

            $requestPath = $parentPath . $urlKey . $categoryUrlSuffix;

            // PATCH TO FIX MAGENTO BUG THAT DUPLICATES REWRITES WHEN URL_KEY IS EQUAL BETWEEN CATEGORIES
            $pattern = '/^' . preg_quote($parentPath.$urlKey, '/') . '-\d{1,}' . preg_quote($categoryUrlSuffix, '/')  . '$/';
            if(isset($existingRequestPath) && (preg_match($pattern, $existingRequestPath) || $requestPath === $existingRequestPath)) {
                return $existingRequestPath;
            }
            // END OF PATCH

            if (isset( $existingRequestPath ) && $existingRequestPath == $requestPath . $suffix) {
                return $existingRequestPath;
            }

            if ($this->_deleteOldTargetPath( $requestPath, $idPath, $storeId )) {
                return $requestPath;
            }

            return $this->getUnusedPath($category->getStoreId(), $requestPath,
                                        $this->generatePath( 'id', null, $category )
            );
        }
    }

    /**
     * Get unique product request path
     *
     * @param   Varien_Object $product
     * @param   Varien_Object $category
     * @return  string
     */
    public function getProductRequestPath($product, $category)
    {
        if ( version_compare( Mage::getVersion(), '1.7.0.0', '<' ) ) {
            // The fix below is not compatible with older Magento versions.
            // Then only the shell command of this extension is useful.
            return parent::getProductRequestPath($product, $category);
        } else {
            if ($product->getUrlKey() == '') {
                $urlKey = $this->getProductModel()->formatUrlKey( $product->getName() );
            } else {
                $urlKey = $this->getProductModel()->formatUrlKey( $product->getUrlKey() );
            }
            $storeId = $category->getStoreId();
            $suffix  = $this->getProductUrlSuffix( $storeId );
            $idPath  = $this->generatePath( 'id', $product, $category );
            /**
             * Prepare product base request path
             */
            if ($category->getLevel() > 1) {
                // To ensure, that category has path either from attribute or generated now
                $this->_addCategoryUrlPath( $category );
                $categoryUrl = Mage::helper('catalog/category')->getCategoryUrlPath($category->getUrlPath(),
                                                                                    false, $storeId);
                $requestPath = $categoryUrl . '/' . $urlKey;
            } else {
                $requestPath = $urlKey;
            }

            if (strlen( $requestPath ) > self::MAX_REQUEST_PATH_LENGTH + self::ALLOWED_REQUEST_PATH_OVERFLOW) {
                $requestPath = substr( $requestPath, 0, self::MAX_REQUEST_PATH_LENGTH );
            }

            $this->_rewrite = null;
            /**
             * Check $requestPath should be unique
             */
            if (isset( $this->_rewrites[ $idPath ] )) {
                $this->_rewrite = $this->_rewrites[ $idPath ];
                $existingRequestPath = $this->_rewrites[ $idPath ]->getRequestPath();

                if ($existingRequestPath == $requestPath . $suffix) {
                    return $existingRequestPath;
                }

                $existingRequestPath = preg_replace('/' . preg_quote($suffix, '/') . '$/', '', $existingRequestPath);
                /**
                 * Check if existing request past can be used
                 */
                if ( $product->getUrlKey() == '' && !empty($requestPath)
                     && strpos($existingRequestPath, $requestPath) === 0 ) {
                    $existingRequestPath = preg_replace(
                    '/^' . preg_quote($requestPath, '/') . '/', '', $existingRequestPath
                    );
                    if (preg_match( '#^-([0-9]+)$#i', $existingRequestPath )) {
                        return $this->_rewrites[ $idPath ]->getRequestPath();
                    }
                }

                $fullPath = $requestPath . $suffix;

                // PATCH TO FIX MAGENTO BUG THAT DUPLICATES REWRITES WHEN URL_KEY IS EQUAL BETWEEN PRODUCTS
                $pattern = '/^' . preg_quote( $requestPath, '/' ) . '-\d{1,}$/';
                if (preg_match( $pattern, $existingRequestPath )) {
                    $fullPath = $existingRequestPath . $suffix;
                }
                // END OF PATCH

                if ($this->_deleteOldTargetPath( $fullPath, $idPath, $storeId )) {
                    return $fullPath;
                }
            }
            /**
             * Check 2 variants: $requestPath and $requestPath . '-' . $productId
             */
            $validatedPath = $this->getResource()->checkRequestPaths(
                array( $requestPath . $suffix, $requestPath . '-' . $product->getId() . $suffix ),
                $storeId
            );

            if ($validatedPath) {
                return $validatedPath;
            }
            /**
             * Use unique path generator
             */
            return $this->getUnusedPath( $storeId, $requestPath . $suffix, $idPath );
        }
    }

}
